/*******************************************************************
 *
 * Name:	BSVideoPluginMotif.c
 *
 * Description: Audio plugin for OpenDVE, H263/H261 compression,
 *
 * Author:	Janusz Bulawa & Tomasz Stachowiak
 *
 * Date:	16/09/96
 *
 * Change Log:
 *
 ********************************************************************/

#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <signal.h>
#include <netdb.h>
#include <netinet/in.h>

#include <Xm/Xm.h>	    /* Motif includes */
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>
#include <Xm/Scale.h>
#include <Xm/MainW.h>
#include <Xm/ToggleB.h>
#include <Xm/Text.h>
#include <Xm/DialogS.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>

#include <dve/Xm/recipientList.h>


/* global variables */
XtAppContext app;

Widget toplevel, shell=NULL;
Widget status_panel, start_but, stop_but;

char *localName;

void tool_menu();
void PanelDialog();

/*
 * Main()
 */
void
main(int argc, char **argv)
{

  Widget toprowcol, rowcol,
	 main_w, menubar, option_menu;
  void start_pushed();
  void stop_pushed();
  void scale_change();
  void file_menu(Widget menu_item, int item_no,
		  XmAnyCallbackStruct *cbs);
  XmString label, str1, str2;



  /*
   * Motif interface initialization
   */

/* Application */
  toplevel = XtVaAppInitialize(&app, "Video tool", NULL, 0, 
			       &argc, argv, NULL,
			       XmNiconName, "Video tool",
			       NULL);
  
/* Main window */
  main_w =  XtVaCreateManagedWidget("main_w", xmMainWindowWidgetClass, 
				    toplevel,
				    NULL);
/* Menu bar */
  str1 = XmStringCreateSimple("File");
  str2 = XmStringCreateSimple("Tool");
  menubar = XmVaCreateSimpleMenuBar(main_w, "menubar",
	    XmVaCASCADEBUTTON, str1, 'F',
	    XmVaCASCADEBUTTON, str2, 'T',
	    NULL);
  XmStringFree(str1);
  XmStringFree(str2);
				    
/* Pulldown menu (File) */
  str1 = XmStringCreateSimple("Exit");
  str2 = XmStringCreateSimple("Ctrl+C");
  XmVaCreateSimplePulldownMenu(menubar, "file_menu", 0, file_menu,
	    XmVaPUSHBUTTON, str1, 'E', "Ctrl<Key>c", str2,
	    NULL);
  XmStringFree(str1);
  XmStringFree(str2);

/* Pulldown menu (Tool) */
  str1 = XmStringCreateSimple("Video panel");
  XmVaCreateSimplePulldownMenu(menubar, "tool_menu", 1, tool_menu,
	    XmVaPUSHBUTTON, str1, 'V', NULL, NULL,
	    NULL);
  XmStringFree(str1);

  XtManageChild(menubar);
 

/* Vertical rowcolum widget */
  toprowcol =  XtVaCreateManagedWidget("toprowcol", xmRowColumnWidgetClass, 
				      main_w,
				      XmNorientation, XmVERTICAL,
				      NULL);

/* Horizontal rowcolumn widget (for buttons) */
  rowcol =  XtVaCreateWidget("rowcol", xmRowColumnWidgetClass, 
				      toprowcol,
				      XmNorientation, XmHORIZONTAL,
				      NULL);
/* Start button */  
  label = XmStringCreateSimple("START");
  start_but = XtVaCreateManagedWidget("start", xmPushButtonWidgetClass, rowcol,
				      XmNwidth, 100,
				      XmNalignment, XmALIGNMENT_CENTER,
				      XmNlabelString, label,
				      NULL);
  XmStringFree(label);

/* Stop button */
  label = XmStringCreateSimple("STOP");
  stop_but = XtVaCreateManagedWidget("stop", xmPushButtonWidgetClass, rowcol,
				     XmNlabelString, label,
				     NULL);
  XmStringFree(label);
  XtSetSensitive(stop_but, False);


/* Buttons callbacks setting */
  XtAddCallback(start_but, XmNactivateCallback, start_pushed, NULL);
  XtAddCallback(stop_but, XmNactivateCallback, stop_pushed, NULL);

  XtManageChild(rowcol);

  status_panel = XtVaCreateManagedWidget("status_panel", xmTextWidgetClass, 
					 toprowcol,
					 XmNvalue, "",
					 XmNautoShowCursorPosition, False,
					 XmNeditable, False,
					 NULL);


 
  XtManageChild(toprowcol);
  XtRealizeWidget(toplevel);

  /*
   * H.261 codec initialization 
   */
  init_codec();
  init_encoder();
  init_decoder();

  /* DVE initialization */
  introduceYourself();

  /* H263 codec initialization */
  initEncoder(176, 144);
  initdecoder();

  XtAppMainLoop(app);
}


SetMessage(const char *msg)
  {
     XmTextSetString(status_panel, msg);
  }






/**********************************************************
 *
 * Video panel dialog 
 *
 *********************************************************/

void v_file_menu();
void comp_option_cb();
void size_option_cb();
void expand_cb();
void intra_cb();

void PanelDialog()
  {
     Widget v_main_w, v_menubar, panel, frame1, frame2, label1, label2,
            sendpanel, recpanel, comp_option_menu, size_option_menu,
            toggle, intra_toggle;
     XmString str1, str2, str3;

     /* main dialog shell */
     shell = XtVaCreatePopupShell("shell", xmDialogShellWidgetClass, 
				  toplevel,
				  XmNtitle, "Video panel",
				  XmNx, 200,
				  XmNy, 200,
				  XmNbaseWidth, 310,
				  XmNbaseHeight, 250, 
				  XmNheightInc, 0, 
				  XmNwidthInc, 0, 
/*				  XmNdeleteResponse, XmDESTROY, */
				  NULL);


     /* Main window */
     v_main_w =  XtVaCreateManagedWidget("v_main_w", xmMainWindowWidgetClass, 
				       shell,
				       NULL);

     /* Menu bar */
     str1 = XmStringCreateSimple("File");
     v_menubar = XmVaCreateSimpleMenuBar(v_main_w, "w_menubar",
				       XmVaCASCADEBUTTON, str1, 'F',
				       NULL);
     XmStringFree(str1);
     
    /* Pulldown menu (File) */
     str1 = XmStringCreateSimple("Close");
     XmVaCreateSimplePulldownMenu(v_menubar, "v_file_menu", 0, v_file_menu,
			   XmVaPUSHBUTTON, str1, 'C', NULL, NULL,
			   NULL);
     XmStringFree(str1);

     XtManageChild(v_menubar);

     /* main dialog panel (rowcol) */
     panel =  XtVaCreateWidget("panel", xmRowColumnWidgetClass, 
			       v_main_w,
			       XmNorientation, XmVERTICAL,
			       NULL);


     /********************************** 
      * first frame (sending options) 
      *********************************/
     frame1 =  XtVaCreateWidget("frame1", xmFrameWidgetClass, 
			       panel,
			       XmNshadowType, XmSHADOW_IN,
			       XmNheight, 100,
			       NULL);
     
     sendpanel =  XtVaCreateWidget("sendpanel", xmRowColumnWidgetClass, 
			       frame1,
			       XmNorientation, XmVERTICAL,
			       NULL);

     label1 = XtVaCreateManagedWidget("Video sending", xmLabelWidgetClass, 
			       sendpanel,
			       NULL);

     str3 = XmStringCreateSimple("Compression:");
     str1 = XmStringCreateSimple("H263");
     str2 = XmStringCreateSimple("H261");
     comp_option_menu = XmVaCreateSimpleOptionMenu(sendpanel, 
		  "comp_option_menu", 
		  str3, 'C', 0, comp_option_cb,
		  XmVaPUSHBUTTON, str1, '3', NULL, NULL,
		  XmVaPUSHBUTTON, str2, '1', NULL, NULL,
		  NULL);
     XmStringFree(str1);
     XmStringFree(str2);
     XmStringFree(str3);

     XtManageChild(comp_option_menu);

     str3 = XmStringCreateSimple("Size:       ");
     str1 = XmStringCreateSimple("QCIF");
     str2 = XmStringCreateSimple("CIF");
     size_option_menu = XmVaCreateSimpleOptionMenu(sendpanel, 
		  "size_option_menu", 
		  str3, 'S', 0, size_option_cb,
		  XmVaPUSHBUTTON, str1, 'Q', NULL, NULL,
		  XmVaPUSHBUTTON, str2, 'C', NULL, NULL,
		  NULL);
     XmStringFree(str1);
     XmStringFree(str2);
     XmStringFree(str3);

     XtManageChild(size_option_menu);

     XtSetSensitive(size_option_menu, False);

     str1 = XmStringCreateSimple("H261 intra frames only");
     intra_toggle = XtVaCreateManagedWidget("intra_toggle", 
			       xmToggleButtonWidgetClass, 
			       sendpanel,
			       XmNindicatorOn, True,
			       XmNlabelString, str1,
			       NULL);
     XmStringFree(str1);

     XtAddCallback(intra_toggle, XmNvalueChangedCallback, intra_cb, NULL);

     XtManageChild(sendpanel);
     XtManageChild(frame1);



     /************************************* 
      * second frame (receiving options) 
      *************************************/

     frame2 =  XtVaCreateWidget("frame2", xmFrameWidgetClass, 
			       panel,
			       XmNshadowType, XmSHADOW_IN,
			       NULL);
     
     recpanel =  XtVaCreateWidget("recpanel", xmRowColumnWidgetClass, 
			       frame2,
			       XmNorientation, XmVERTICAL,
			       NULL);

     label2 = XtVaCreateManagedWidget("Video receiving", xmLabelWidgetClass, 
			       recpanel,
			       NULL);

     str1 = XmStringCreateSimple("Expand");
     toggle = XtVaCreateManagedWidget("Toggle", xmToggleButtonWidgetClass, 
			       recpanel,
			       XmNindicatorOn, True,
			       XmNlabelString, str1,
			       NULL);
     XmStringFree(str1);

     XtAddCallback(toggle, XmNvalueChangedCallback, expand_cb, NULL);


     XtManageChild(recpanel);
     XtManageChild(frame2);
     XtManageChild(panel);
     XtManageChild(shell);

     XtRealizeWidget(shell);

     XtPopup(shell, XtGrabNone);
  }

